Package: mcglm
Type: Package
Title: Multivariate Covariance Generalized Linear Models
Version: 0.8.0
Date: 2022-09-15
Author: Wagner Hugo Bonat [aut, cre]
Maintainer: Wagner Hugo Bonat <wbonat@ufpr.br>
Authors@R: c(person(c("Wagner","Hugo"), "Bonat", role = c("aut", "cre"),
                    email = "wbonat@ufpr.br"))
Description: Fitting multivariate covariance generalized linear
     models (McGLMs) to data.  McGLM is a general framework for non-normal
     multivariate data analysis, designed to handle multivariate response
     variables, along with a wide range of temporal and spatial correlation
     structures defined in terms of a covariance link function combined
     with a matrix linear predictor involving known matrices.
     The models take non-normality into account in the conventional way
     by means of a variance function, and the mean structure is modelled
     by means of a link function and a linear predictor.
     The models are fitted using an efficient Newton scoring algorithm
     based on quasi-likelihood and Pearson estimating functions, using
     only second-moment assumptions. This provides a unified approach to
     a wide variety of different types of response variables and covariance
     structures, including multivariate extensions of repeated measures,
     time series, longitudinal, spatial and spatio-temporal structures.
     The package offers a user-friendly interface for fitting McGLMs
     similar to the glm() R function. 
     See Bonat (2018) <doi:10.18637/jss.v084.i04>, for more information 
     and examples.
Depends: R (>= 4.2.0)
Suggests: testthat, knitr, rmarkdown, MASS, mvtnorm, tweedie, devtools
Imports: stats, Matrix, assertthat, graphics, Rcpp (>= 0.12.16)
License: GPL-3 | file LICENSE
LazyData: TRUE
URL: mcglm.leg.ufpr.br
BugReports: https://github.com/wbonat/mcglm/issues
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.2.0
LinkingTo: Rcpp, RcppArmadillo
NeedsCompilation: yes
Packaged: 2022-09-15 18:27:11 UTC; root
Repository: CRAN
Date/Publication: 2022-09-15 19:36:07 UTC
Built: R 4.4.1; x86_64-apple-darwin24.0.0; 2024-10-15 08:14:09 UTC; unix
