# Generated by roxygen2: do not edit by hand

S3method(anova,mcglm)
S3method(coef,mcglm)
S3method(confint,mcglm)
S3method(fitted,mcglm)
S3method(plot,mcglm)
S3method(print,mcglm)
S3method(residuals,mcglm)
S3method(summary,mcglm)
S3method(vcov,mcglm)
export(ESS)
export(GOSHO)
export(RJC)
export(fit_mcglm)
export(gof)
export(mc_anova_disp)
export(mc_bias_corrected_std)
export(mc_car)
export(mc_complete_data)
export(mc_compute_rho)
export(mc_conditional_test)
export(mc_dexp_gold)
export(mc_dglm)
export(mc_dist)
export(mc_id)
export(mc_initial_values)
export(mc_link_function)
export(mc_ma)
export(mc_manova)
export(mc_manova_disp)
export(mc_matrix_linear_predictor)
export(mc_mixed)
export(mc_ns)
export(mc_quasi_score)
export(mc_remove_na)
export(mc_robust_std)
export(mc_rw)
export(mc_sic)
export(mc_sic_covariance)
export(mc_twin)
export(mc_variance_function)
export(mcglm)
export(pAIC)
export(pBIC)
export(pKLIC)
export(plogLik)
import(Matrix)
import(assertthat)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,dev.new)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,dist)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,make.link)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.exclude)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quasi)
importFrom(stats,relevel)
importFrom(stats,residuals)
importFrom(stats,vcov)
importFrom(utils,combn)
useDynLib(mcglm)
